!     %--------------------------------%
!     | See stat.doc for documentation |
!     %--------------------------------%
!
!\SCCS Information: @(#) 
! FILE: stat.h   SID: 2.2   DATE OF SID: 11/16/95   RELEASE: 2 
!
      real       t0, t1, t2, t3, t4, t5
      save       t0, t1, t2, t3, t4, t5
!
      integer    nopx, nbx, nrorth, nitref, nrstrt
      real       tsaupd, tsaup2, tsaitr, tseigt, tsgets, tsapps, tsconv,&
                 tnaupd, tnaup2, tnaitr, tneigh, tngets, tnapps, tnconv,&
                 tcaupd, tcaup2, tcaitr, tceigh, tcgets, tcapps, tcconv,&
                 tmvopx, tmvbx, tgetv0, titref, trvec
      common /timing/                                                   &
                 nopx, nbx, nrorth, nitref, nrstrt,                     &
                 tsaupd, tsaup2, tsaitr, tseigt, tsgets, tsapps, tsconv,&
                 tnaupd, tnaup2, tnaitr, tneigh, tngets, tnapps, tnconv,&
                 tcaupd, tcaup2, tcaitr, tceigh, tcgets, tcapps, tcconv,&
                 tmvopx, tmvbx, tgetv0, titref, trvec
