!
!\SCCS Information: @(#)
! FILE: debug.h   SID: 2.3   DATE OF SID: 11/16/95   RELEASE: 2
!
!     %---------------------------------%
!     | See debug.doc for documentation |
!     %---------------------------------%
      integer  logfil, ndigit, mgetv0,                                &
               msaupd, msaup2, msaitr, mseigt, msapps, msgets, mseupd,&
               mnaupd, mnaup2, mnaitr, mneigh, mnapps, mngets, mneupd,&
               mcaupd, mcaup2, mcaitr, mceigh, mcapps, mcgets, mceupd
      common /debug/                                                  &
               logfil, ndigit, mgetv0,                                &
               msaupd, msaup2, msaitr, mseigt, msapps, msgets, mseupd,&
               mnaupd, mnaup2, mnaitr, mneigh, mnapps, mngets, mneupd,&
               mcaupd, mcaup2, mcaitr, mceigh, mcapps, mcgets, mceupd
