LIBRARY    lapack.dll


EXPORTS
dlamch_
dlapy2_
zlaset_
zlahqr_
ztrsen_
zgeqr2_
zunm2r_
zlacpy_
ztrevc_
dlabad_
dlaset_
dlanhs_
dlartg_
dlarfg_
dlarf_
dlarnv_
dlaruv_
dlacpy_
dswap_
dlascl_
dtrevc_
dlanst_
dlaev2_
dlae2_
dlasr_
dlasrt_
zlascl_
zlanhs_
dsteqr_
dgeqr2_
dorm2r_
dlahqr_
dtrsen_
dlanv2_
zlarnv_
zlartg_
;
slamch_
slapy2_
cswap_
slabad_
claset_
slarf_
strmm_
strevc_
strsen_
slahqr_
saxpy_
sger_
slacpy_
sorm2r_
sgeqr2_
ssteqr_
slaset_
slasrt_
slartg_
slae2_
slasr_
slaev2_
slascl_
slanst_
sscal_
clarnv_
sswap_
cgeru_
ctrmm_
ctrevc_
clanhs_
clartg_
cscal_
cgemv_
ccopy_
clacpy_
caxpy_
slanhs_
scopy_
slarfg_
slanv2_
srot_
csscal_
clascl_
cdotc_
scnrm2_
slarnv_
sdot_
snrm2_
sgemv_
clahqr_
ctrsen_
cgeqr2_
cunm2r_




